/////////////////////////////////////////////////////////////////////////////////
// bigbench - small benchmark :)
//
// written by S.Cocquereaumont for int13 production
//
//
/////////////////////////////////////////////////////////////////////////////////

/// includes ////////////////////////////////////////////////////////////////////

/// constants ///////////////////////////////////////////////////////////////////

#define MAXBLITS 		5000
#define SIZE_SMALLSURFACE 	32
#define SIZE_LARGESURFACE 	128

#define SP32			10	// sprite 32*32 colorkeyed
#define SP128			11 	// sprite 128*128 colorkeyed
#define SP32A			12	// sprite 32*32 with alpha channel
#define SP128A			13	// sprite 128*128 with alpha channel

/// structures //////////////////////////////////////////////////////////////////

typedef struct
{
  Uint16 x;
  Uint16 y;
} XYPoint;

/// globals /////////////////////////////////////////////////////////////////////

XYPoint RandMap[MAXBLITS];

/// macros //////////////////////////////////////////////////////////////////////

/// headers /////////////////////////////////////////////////////////////////////

void InitBench();
void RunBench();

/// code ////////////////////////////////////////////////////////////////////////

void StartBench()
{
  Uint32 i;

  Log("---------------------------");
  Log(" uEngine 2D benchmark v1.0");
  Log("---------------------------");

  Log(" >> Init");

  CLEARSCREEN(0); UPDATESCREEN;

  // init phase
  InitBench();

  ToggleLog(1);

  Log(" >> Ready");
  Log("---------------------------");
  Log(" Press any key to start...");

  CLEARSCREEN(0); UPDATESCREEN;

  for(i = 0; i< 20; i++) { CLEARSCREEN(0); UPDATESCREEN; }

  WAITANYKEY;

  // run the actual bench
  RunBench();

  CLEARSCREEN(0);
  UPDATESCREEN;

  WAITANYKEY;

  // save the results in a file (log.txt)
  SaveLog();

  // and exit
  return;
}

/////////////////////////////////////////////////////////////////////////////////

void InitBench()
{
  Uint32 i;

  // load sprites

  LoadSprites("sprite32x32.tga", SP32, 1);
  LoadSprites("sprite32x32alpha.tga", SP32A, 1);
  LoadSprites("sprite128x128.tga", SP128, 1);
  LoadSprites("sprite128x128alpha.tga", SP128A, 1);

  // init the random map

  srand(0);
  for(i = 0; i < MAXBLITS; i++)
  {
    RandMap[i].x = rand() % (240 - 128);
    RandMap[i].y = rand() % (240 - 128);
  }
}

/////////////////////////////////////////////////////////////////////////////////

void RunBench()
{
  Uint32 i;

  Log("---------------------------");

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { DrawSprite(SP32, RandMap[i].x, RandMap[i].y); }
  Log("BltFastColorkey 32x32 : %d ms", Profile(1));
  CLEARSCREEN(0); UPDATESCREEN;

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { DrawSpriteAlpha(SP32, RandMap[i].x, RandMap[i].y, 16); }
  Log("BltFastColorkey 32x32 50%% opacity : %d ms", Profile(1));
  CLEARSCREEN(0); UPDATESCREEN;

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { DrawSpriteAlpha(SP32, RandMap[i].x, RandMap[i].y, 3); }
  Log("BltFastColorkey 32x32 10%% opacity : %d ms", Profile(1));
  CLEARSCREEN(0); UPDATESCREEN;

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { DrawSprite(SP128, RandMap[i].x, RandMap[i].y); }
  Log("BltFastColorkey 128x128 : %d ms", Profile(1));

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { DrawSpriteAlpha(SP128, RandMap[i].x, RandMap[i].y, 16); }
  Log("BltFastColorkey 128x128 50%% opacity : %d ms", Profile(1));

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { DrawSpriteAlpha(SP128, RandMap[i].x, RandMap[i].y, 3); }
  Log("BltFastColorkey 128x128 10%% opacity : %d ms", Profile(1));

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { DrawSprite(SP32A, RandMap[i].x, RandMap[i].y); }
  Log("AlphaBltFast 32x32 : %d ms", Profile(1));
  CLEARSCREEN(0); UPDATESCREEN;

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { DrawSprite(SP128A, RandMap[i].x, RandMap[i].y); }
  Log("AlphaBltFast 128x128 : %d ms", Profile(1));

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { DrawFilledRect(XYPOINT(RandMap[i].x, RandMap[i].y), 32, 32, RGB2COLOR(0,0,255)); }
  Log("FillRect 32x32 : %d ms", Profile(1));
  CLEARSCREEN(0);UPDATESCREEN;

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { AlphaBlendRect(RandMap[i].x, RandMap[i].y, 32, 32, RGB2COLOR(0,0,255), 16); }
  Log("FillRect 32x32 50%% opacity : %d ms", Profile(1));
  CLEARSCREEN(0);UPDATESCREEN;

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { AlphaBlendRect(RandMap[i].x, RandMap[i].y, 32, 32, RGB2COLOR(0,0,255), 3); }
  Log("FillRect 32x32 10%% opacity : %d ms", Profile(1));
  CLEARSCREEN(0);UPDATESCREEN;

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { DrawFilledRect(XYPOINT(RandMap[i].x, RandMap[i].y), 128, 128, RGB2COLOR(0,0,255)); }
  Log("FillRect 128x128 : %d ms", Profile(1));
  CLEARSCREEN(0);UPDATESCREEN;

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { AlphaBlendRect(RandMap[i].x, RandMap[i].y, 128, 128, RGB2COLOR(0,0,255), 16); }
  Log("FillRect 128x128 50%% opacity : %d ms", Profile(1));
  CLEARSCREEN(0);UPDATESCREEN;

  Profile(0);
  for(i = 0; i < MAXBLITS; i++) { AlphaBlendRect(RandMap[i].x, RandMap[i].y, 128, 128, RGB2COLOR(0,0,255), 3); }
  Log("FillRect 128x128 10%% opacity : %d ms", Profile(1));

  Log("---------------------------");
  Log(" Bench complete !");
  Log("---------------------------");
  Log(" Press any key to exit...");

}


/// end of file /////////////////////////////////////////////////////////////////
